@@ -68,6 +68,27 @@ static void GetBitmapFile(FvwmWindow *tmp_win);
 static void GetXPMFile(FvwmWindow *tmp_win);
 #endif
 
+int get_visible_icon_window_count(FvwmWindow *tmp_win)
+{
+	int count = 0;
+
+	if (tmp_win == NULL || !IS_ICONIFIED(tmp_win) ||
+	    IS_ICON_SUPPRESSED(tmp_win))
+	{
+		return 0;
+	}
+	if (tmp_win->icon_pixmap_w != None)
+	{
+		count++;
+	}
+	if (tmp_win->icon_w != None)
+	{
+		count++;
+	}
+
+	return count;
+}
+
 /****************************************************************************
  *
  * Creates an icon window as needed
@@ -217,7 +238,8 @@ ICON_DBG((stderr,"ciw: iph%s used '%s'\n", (tmp_win->icon_p_height)?"":" not",tm
   /* this does not happen if fvwm is using a non-default visual (with
      private colormap) and the client has supplied a pixmap (not a bitmap) */
   if ((IS_ICON_OURS(tmp_win)) && (tmp_win->icon_p_height > 0)
-      && (Pdefault || (tmp_win->iconDepth == 1) || IS_PIXMAP_OURS(tmp_win)))
+      && (tmp_win->iconDepth == Pdepth || tmp_win->iconDepth == 1 ||
+	  IS_PIXMAP_OURS(tmp_win)))
   {
     tmp_win->icon_p_width += 4;
     tmp_win->icon_p_height += 4;
@@ -273,7 +295,8 @@ ICON_DBG((stderr,"ciw: iph%s used '%s'\n", (tmp_win->icon_p_height)?"":" not",tm
       tmp_win->icon_p_height > 0)
   {
     /* use fvwm's visuals in these cases */
-    if (Pdefault || (tmp_win->iconDepth == 1) || IS_PIXMAP_OURS(tmp_win))
+    if (tmp_win->iconDepth == Pdepth || tmp_win->iconDepth == 1 ||
+	IS_PIXMAP_OURS(tmp_win))
     {
       if (!old_icon_pixmap_w)
       {
@@ -320,6 +343,10 @@ ICON_DBG((stderr,"ciw: iph%s used '%s'\n", (tmp_win->icon_p_height)?"":" not",tm
     attributes.event_mask = XEVMASK_ICONPW;
     valuemask = CWEventMask;
     XChangeWindowAttributes(dpy, tmp_win->icon_pixmap_w, valuemask,&attributes);
+    if (!IS_ICON_OURS(tmp_win))
+    {
+      XMoveWindow(dpy, tmp_win->icon_pixmap_w, def_x, def_y);
+    }
   }
   if (old_icon_pixmap_w && old_icon_pixmap_w != tmp_win->icon_pixmap_w)
   {
@@ -336,7 +363,8 @@ ICON_DBG((stderr,"ciw: iph%s used '%s'\n", (tmp_win->icon_p_height)?"":" not",tm
       /* when fvwm is using the non-default visual client supplied icon pixmaps
        * are drawn in a window with no relief */
       int off;
-      off = (Pdefault || (tmp_win->iconDepth == 1) || IS_PIXMAP_OURS(tmp_win)) ?
+      off = (tmp_win->iconDepth == Pdepth || tmp_win->iconDepth == 1 ||
+	     IS_PIXMAP_OURS(tmp_win)) ?
 	2 : 0;
       FShapeCombineMask(
 	dpy, tmp_win->icon_pixmap_w, FShapeBounding, off, off,
@@ -540,7 +568,8 @@ void DrawIconWindow(FvwmWindow *tmp_win)
 
   /* only relieve unshaped icons that share fvwm's visual */
   if ((tmp_win->iconPixmap != None) && !IS_ICON_SHAPED(tmp_win)
-      && (Pdefault || (tmp_win->iconDepth == 1) || IS_PIXMAP_OURS(tmp_win)))
+      && (tmp_win->iconDepth == Pdepth || tmp_win->iconDepth == 1 ||
+	  IS_PIXMAP_OURS(tmp_win)))
     RelieveRectangle(dpy, tmp_win->icon_pixmap_w, 0, 0,
 		     tmp_win->icon_p_width - 1, tmp_win->icon_p_height - 1,
 		     Relief, Shadow, 2);
@@ -557,7 +586,7 @@ void DrawIconWindow(FvwmWindow *tmp_win)
     }
     else
     {
-      if (Pdefault || IS_PIXMAP_OURS(tmp_win))
+      if (tmp_win->iconDepth == Pdepth || IS_PIXMAP_OURS(tmp_win))
       {
         /* it's a pixmap that need copying */
 	XCopyArea(dpy, tmp_win->iconPixmap, tmp_win->icon_pixmap_w,
@@ -722,6 +751,8 @@ void AutoPlaceIcon(FvwmWindow *t)
     t->icon_g.x = t->wmhints->icon_x;
     t->icon_g.y = t->wmhints->icon_y;
     t->icon_xl_loc = t->icon_g.x;
+    do_move_pixmap = True;
+    do_move_title = True;
   }
   /* dje 10/12/97:
      Look thru chain of icon boxes assigned to window.
@@ -1148,8 +1179,14 @@ static void GetXPMFile(FvwmWindow *tmp_win)
   rc = XpmReadFileToXpmImage(path, &my_image, NULL);
   if (rc != XpmSuccess)
   {
-    fvwm_msg(ERR,"GetXPMFile","XpmReadFileToXpmImage failed, pixmap %s, rc %d",
-	     path, rc);
+    int len = strlen(path);
+
+    /* no message for files that don't have an 'xpm' suffix */
+    if (len >= 3 && StrEquals(path + len - 3, "xpm"))
+    {
+      fvwm_msg(ERR, "GetXPMFile",
+         "XpmReadFileToXpmImage failed, pixmap %s, rc %d", path, rc);
+    }
     free(path);
     return;
   }
@@ -1188,12 +1225,18 @@ void GetIconWindow(FvwmWindow *tmp_win)
 {
   /* We are guaranteed that wmhints is non-null when calling this
    * routine */
-  if(XGetGeometry(dpy,   tmp_win->wmhints->icon_window, &JunkRoot,
-		  &JunkX, &JunkY,(unsigned int *)&tmp_win->icon_p_width,
-		  (unsigned int *)&tmp_win->icon_p_height,
-		  &JunkBW, &JunkDepth)==0)
+  if (XGetGeometry(
+	dpy, tmp_win->wmhints->icon_window, &JunkRoot, &JunkX, &JunkY,
+	(unsigned int *)&tmp_win->icon_p_width,
+	(unsigned int *)&tmp_win->icon_p_height, &JunkBW, &JunkDepth) == 0)
   {
-    fvwm_msg(ERR,"GetIconWindow","Help! Bad Icon Window!");
+    fvwm_msg(ERR,"GetIconWindow", "Window '%s' has a bad icon window!"
+	     " Ignoring icon window.",
+	     tmp_win->name);
+    /* disable the icon window hint */
+    tmp_win->wmhints->icon_window = None;
+    tmp_win->wmhints->flags &= ~IconWindowHint;
+    return;
   }
   tmp_win->icon_p_width += JunkBW<<1;
   tmp_win->icon_p_height += JunkBW<<1;
@@ -1232,13 +1275,24 @@ void GetIconBitmap(FvwmWindow *tmp_win)
   if (!XGetGeometry(dpy, tmp_win->wmhints->icon_pixmap, &JunkRoot,
 		    &JunkX, &JunkY, &width, &height, &JunkBW, &depth))
   {
+    fvwm_msg(ERR,"GetIconBitmap", "Window '%s' has a bad icon pixmap!"
+	     " Ignoring icon.", tmp_win->name);
+    /* disable icon pixmap hint */
+    tmp_win->wmhints->icon_pixmap = None;
+    tmp_win->wmhints->flags &= ~IconPixmapHint;
     return;
   }
 
   /* sanity check the pixmap depth, it must be the same as the root or 1 */
-  if ((depth != 1) && (depth != DefaultDepth(dpy,Scr.screen)))
+  if (depth != 1 && depth != Pdepth && depth != DefaultDepth(dpy,Scr.screen))
   {
-    fvwm_msg(ERR, "GetIconBitmap", "Bad client icon pixmap depth %d", depth);
+    fvwm_msg(ERR, "GetIconBitmap",
+	     "Window '%s' has a bad icon bitmap depth %d "
+	     "(should be 1, %d or %d)! Ignoring icon bitmap.",
+	     tmp_win->name, depth, Pdepth, DefaultDepth(dpy,Scr.screen));
+    /* disable icon pixmap hint */
+    tmp_win->wmhints->icon_pixmap = None;
+    tmp_win->wmhints->flags &= ~IconPixmapHint;
     return;
   }
 
@@ -1291,12 +1345,14 @@ void DeIconify(FvwmWindow *tmp_win)
 
   /* AS dje  RaiseWindow(tmp_win); */
 
-  mark_transient_subtree(tmp_win, MARK_ALL_LAYERS, MARK_ALL, False, True);
   if (tmp_win == sf)
   {
     /* take away the focus before mapping */
     DeleteFocus(0);
   }
+  /* Note: DeleteFocus may delete the flags set by mark_transient_subtree(),
+   * so do it later. */
+  mark_transient_subtree(tmp_win, MARK_ALL_LAYERS, MARK_ALL, False, True);
   /* now de-iconify transients */
   for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
   {
@@ -1454,6 +1510,9 @@ void Iconify(FvwmWindow *tmp_win, int def_x, int def_y)
   if (sf && IS_IN_TRANSIENT_SUBTREE(sf))
   {
     restore_focus_after_unmap(sf, True);
+    /* restore_focus_after_unmap() destorys the flags set by
+     * mark_transient_subtree(), so we have to unfortunately call it again. */
+    mark_transient_subtree(tmp_win, MARK_ALL_LAYERS, MARK_ALL, False, True);
   }
   /* iconify transients first */
   for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
