@@ -61,8 +61,7 @@ static int push_read_file(const char *file)
     return FALSE;
   }
   prev_read_files[curr_read_depth++] = curr_read_file;
-  /* we need a library function safestrdup(); stripcpy() is bad for this */
-  curr_read_file = strdup(file);
+  curr_read_file = safestrdup(file);
 
   if (curr_read_dir)
     free(curr_read_dir);
@@ -136,12 +135,16 @@ void run_command_stream( FILE* f, XEvent *eventp, FvwmWindow *tmp_win,
     tline=line;
     while(isspace((unsigned char)*tline))
       tline++;
+    l = strlen(tline);
+    if (l > 0 && tline[l - 1] == '\n')
+      tline[l - 1] = '\0';
+
     if (debugging)
-      fvwm_msg(DBG,"ReadSubFunc","Module switch %d, about to exec: '%.*s'",
-	       Module,strlen(tline)-1,tline);
+      fvwm_msg(DBG, "ReadSubFunc", "Module switch %d, about to exec: '%s'",
+	       Module, tline);
 
     old_execute_function(tline, tmp_win, eventp, context, Module, 0, NULL);
-    tline = fgets(line,(sizeof line)-1,f);
+    tline = fgets(line, (sizeof line) - 1, f);
   }
 }
 
@@ -252,7 +255,7 @@ static void cursor_control(Bool grab)
 
 }
 
-void ReadFile(F_CMD_ARGS)
+void CMD_Read(F_CMD_ARGS)
 {
   char* filename;
   int read_quietly;
@@ -279,7 +282,7 @@ void ReadFile(F_CMD_ARGS)
 
 
 
-void PipeRead(F_CMD_ARGS)
+void CMD_PipeRead(F_CMD_ARGS)
 {
   char* command;
   int read_quietly;
