@@ -42,13 +42,14 @@
 #include <X11/StringDefs.h>
 #include <X11/Intrinsic.h>
 #include <X11/Shell.h>
-#include <X11/extensions/shape.h>
 
 #include <X11/xpm.h>
 
 #include <libs/fvwmlib.h>
 #include <libs/Picture.h>
 #include <libs/Module.h>
+#include <libs/FScreen.h>
+#include <libs/FShape.h>
 
 
 /* migo (16-Sep-1999): How about to do this configurable? */
@@ -85,7 +86,6 @@ Display *dpy;			/* which display are we talking to */
 Window Root;
 int screen;
 int x_fd;
-int ScreenWidth, ScreenHeight;
 XSizeHints mysizehints;
 Pixel back_pix, fore_pix;
 GC NormalGC,FGC;
@@ -140,19 +140,18 @@ int main(int argc, char **argv)
 
   /* Open the display */
   if (!(dpy = XOpenDisplay(display_name)))
-    {
-      fprintf(stderr,"FvwmBanner: can't open display %s",
-	      XDisplayName(display_name));
-      exit (1);
-    }
+  {
+    fprintf(stderr,"FvwmBanner: can't open display %s",
+	    XDisplayName(display_name));
+    exit (1);
+  }
   screen= DefaultScreen(dpy);
   Root = RootWindow(dpy, screen);
   x_fd = XConnectionNumber(dpy);
 
-  ScreenHeight = DisplayHeight(dpy,screen);
-  ScreenWidth = DisplayWidth(dpy,screen);
-
   InitPictureCMap(dpy);
+  FScreenInit(dpy);
+  FShapeInit(dpy);
   parseOptions(fd);
 
   /* chick in the neck situation:
@@ -190,8 +189,9 @@ int main(int argc, char **argv)
   mysizehints.max_width = mysizehints.width;
   mysizehints.win_gravity = NorthWestGravity;
 
-  mysizehints.x = (ScreenWidth - view.attributes.width)/2;
-  mysizehints.y = (ScreenHeight - view.attributes.height)/2;
+  FScreenCenterOnScreen(
+    NULL, FSCREEN_PRIMARY, &mysizehints.x, &mysizehints.y,
+    view.attributes.width, view.attributes.height);
 
   wm_del_win = XInternAtom(dpy,"WM_DELETE_WINDOW",False);
   XSetWMProtocols(dpy,win,&wm_del_win,1);
@@ -202,10 +202,8 @@ int main(int argc, char **argv)
   XMoveResizeWindow(dpy, win, mysizehints.x, mysizehints.y, mysizehints.width,
 		    mysizehints.height);
 
-#ifdef SHAPE
-  if(view.mask != None)
-    XShapeCombineMask(dpy, win, ShapeBounding,0,0,view.mask, ShapeSet);
-#endif
+  if (FHaveShapeExtension && view.mask != None)
+    FShapeCombineMask(dpy, win, FShapeBounding, 0, 0, view.mask, FShapeSet);
 
   XMapWindow(dpy,win);
   XSync(dpy,0);
@@ -233,25 +231,25 @@ int main(int argc, char **argv)
       exit(0);
     }
 
-    if(FD_ISSET(x_fd, &in_fdset))
+    if (FD_ISSET(x_fd, &in_fdset))
     {
       /* read a packet */
       XNextEvent(dpy,&Event);
       switch(Event.type)
       {
-        case ButtonRelease:
-          XDestroyWindow(dpy,win);
-          XSync(dpy,0);
-          exit(0);
-        case ClientMessage:
-          if (Event.xclient.format==32 && Event.xclient.data.l[0]==wm_del_win)
-          {
-            XDestroyWindow(dpy,win);
-            XSync(dpy,0);
-            exit(0);
-          }
-        default:
-          break;
+      case ButtonRelease:
+	XDestroyWindow(dpy,win);
+	XSync(dpy,0);
+	exit(0);
+      case ClientMessage:
+	if (Event.xclient.format==32 && Event.xclient.data.l[0]==wm_del_win)
+	{
+	  XDestroyWindow(dpy,win);
+	  XSync(dpy,0);
+	  exit(0);
+	}
+      default:
+	break;
       }
     }
   }
@@ -309,42 +307,60 @@ void nocolor(char *a, char *b)
  fprintf(stderr,"FvwmBanner: can't %s %s\n", a,b);
 }
 
-static void parseOptions (int fd[2])
+static void parseOptions(int fd[2])
 {
   char *tline= NULL;
-  char *p;
+  char *tok;
 
   InitGetConfigLine(fd,MyName);
-  while (GetConfigLine (fd, &tline),tline != NULL) {
-    if (strlen (tline) > 1) {
-      if (strncasecmp(tline, "ImagePath",9)==0) {
-        CopyString (&imagePath, &tline[9]);
-        if (imagePath[0] == 0) {
+  while (GetConfigLine (fd, &tline),tline != NULL)
+  {
+    if (strlen(tline) > 1)
+    {
+      tok = PeekToken(tline, &tline);
+      if (StrEquals(tok, XINERAMA_CONFIG_STRING))
+      {
+	FScreenConfigureModule(tline);
+	continue;
+      }
+      if (StrEquals(tok, "ImagePath"))
+      {
+        CopyString(&imagePath, tline);
+        if (imagePath[0] == 0)
+	{
           free (imagePath);
           imagePath = (char *) 0;
         }
         continue;
       }
-      if (strncasecmp(tline,MyName,MyNameLen)) { /* if not for me */
-        continue;                       /* ignore it */
+      if (strncasecmp(tok,MyName,MyNameLen))
+      {
+	/* if not for me: ignore it*/
+        continue;
       }
-      p = tline+MyNameLen;              /* start of interesting part */
-      if (strncasecmp (p, "Pixmap", 6) == 0) {
-	if (imageName == (char *) 0) {
-	  CopyString (&imageName, p+7);
-	  if (imageName[0] == 0) {
+      /* start of interesting part */
+      tok += MyNameLen;
+      if (StrEquals(tok, "Pixmap"))
+      {
+	if (imageName == (char *) 0)
+	{
+	  CopyString(&imageName, tline);
+	  if (imageName[0] == 0)
+	  {
 	    free (imageName);
 	    imageName = (char *) 0;
 	  }
 	}
         continue;
       }
-      if (strncasecmp (p, "NoDecor", 7) == 0) {
+      if (StrEquals(tok, "NoDecor"))
+      {
         no_wm = True;
         continue;
       }
-      if (strncasecmp (p, "Timeout", 7) == 0) {
-        timeout = atoi(p+8) * 1000000;
+      if (StrEquals(tok, "Timeout"))
+      {
+        timeout = atoi(tline) * 1000000;
         continue;
       }
     }
@@ -360,10 +376,10 @@ void change_window_name(char *str)
   XTextProperty name;
 
   if (XStringListToTextProperty(&str,1,&name) == 0)
-    {
-      fprintf(stderr,"FvwmBanner: cannot allocate window name");
-      return;
-    }
+  {
+    fprintf(stderr,"FvwmBanner: cannot allocate window name");
+    return;
+  }
   XSetWMName(dpy,win,&name);
   XSetWMIconName(dpy,win,&name);
   XFree(name.value);
@@ -382,4 +398,3 @@ void DeadPipe (int nonsense)
 {
   exit (0);
 }
-
