@@ -51,9 +51,10 @@
 #include "FvwmM4.h"
 #include "libs/Picture.h"
 #include "libs/fvwmlib.h"
+#include "libs/FShape.h"
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
-#define Resolution(pixels, mm) ((((pixels) * 100000 / (mm)) + 50) / 100)
+#define Resolution(pixels, mm) ((((pixels) * 2000 / (mm)) + 1) / 2)
 
 char *MyName;
 int fd[2];
@@ -63,10 +64,11 @@ static char *MkDef(char *name, char *def);
 static char *MkNum(char *name,int def);
 static char *m4_defs(Display *display, const char *host, char *m4_options, char *config_file);
 #define MAXHOSTNAME 255
-#define EXTRA 50
+#define EXTRA 56
 
 int  m4_enable;                 /* use m4? */
 int  m4_prefix;                 /* Do GNU m4 prefixing (-P) */
+int  m4_prefix_defines;		/* Add "m4_" to the names of the defines */
 char m4_options[BUFSIZ];        /* Command line options to m4 */
 char m4_outfile[BUFSIZ] = "";   /* The output filename for m4 */
 char *m4_prog = "m4";           /* Name of the m4 program */
@@ -93,6 +95,7 @@ int main(int argc, char **argv)
   int lock = 0;
   int noread = 0;
   char *user_dir;
+  char *m4path;
 
   /* Figure out the working directory and go to it */
   user_dir = getenv("FVWM_USERDIR");
@@ -105,8 +108,28 @@ int main(int argc, char **argv)
 
   m4_enable = True;
   m4_prefix = False;
-  sprintf(m4_options, "-I '%s' ", FVWM_DATADIR);
+  m4_prefix_defines = False;
+  sprintf(m4_options, " ");
   m4_default_quotes = 1;
+  /* add FVWM_DATADIR to the include path.  Can't use the -I option here because
+   * it is incompatible with the System V version of m4.  Instead, append it to
+   * the front of the M4PATH environment variable. */
+  m4path = getenv("M4PATH");
+  if (m4path == NULL)
+  {
+    m4path = safemalloc(sizeof("M4PATH=") + strlen(FVWM_DATADIR) + 1);
+    sprintf(m4path, "M4PATH=%s", FVWM_DATADIR);
+  }
+  else
+  {
+    char *s;
+
+    s = safemalloc(
+      sizeof("M4PATH=") + strlen(FVWM_DATADIR) + strlen(m4path) + 2);
+    sprintf(s, "M4PATH=%s:%s", FVWM_DATADIR, m4path);
+    m4path = s;
+  }
+  putenv(m4path);
 
   /* Record the program name for error messages */
   temp = argv[0];
@@ -138,6 +161,10 @@ int main(int argc, char **argv)
 	{
 	  m4_prefix = TRUE;
 	}
+      else if(strcasecmp(argv[i],"-m4-prefix-defines") == 0)
+	{
+	  m4_prefix_defines = TRUE;
+	}
       else if(strcasecmp(argv[i],"-m4opt") == 0)
 	{
 	  /* leaving this in just in case-- any option starting with '-'
@@ -228,7 +255,7 @@ int main(int argc, char **argv)
   {
     char *delete_string;
     char *delete_file = tmp_file;
-    if (tmp_file[0] != '/' && user_dir != NULL) 
+    if (tmp_file[0] != '/' && user_dir != NULL)
     {
       delete_file = safestrdup(CatString3(user_dir, "/", tmp_file));
     }
@@ -241,7 +268,8 @@ int main(int argc, char **argv)
 
 
 
-static char *m4_defs(Display *display, const char *host, char *m4_options, char *config_file)
+static char *m4_defs(
+  Display *display, const char *host, char *m4_options, char *config_file)
 {
   Screen *screen;
   Visual *visual;
@@ -283,7 +311,7 @@ static char *m4_defs(Display *display, const char *host, char *m4_options, char
   */
   /* first try to unlink it */
   unlink(tmp_name);
-  if ((fd = open(tmp_name, O_WRONLY|O_EXCL|O_CREAT, 0644)) < 0)
+  if ((fd = open(tmp_name, O_WRONLY|O_EXCL|O_CREAT, 0600)) < 0)
   {
     perror("exclusive open for output file failed in m4_defs");
     exit(0377);
@@ -308,7 +336,7 @@ static char *m4_defs(Display *display, const char *host, char *m4_options, char
     perror("Cannot open pipe to m4");
     exit(0377);
   }
-  
+
   gethostname(client,MAXHOSTNAME);
 
   getostype  (ostype, sizeof ostype);
@@ -436,14 +464,13 @@ static char *m4_defs(Display *display, const char *host, char *m4_options, char
 
   /* Add options together */
   *options = '\0';
-#ifdef	SHAPE
-  strcat(options, "SHAPE ");
-#endif
+  if (FHaveShapeExtension)
+    strcat(options, "SHAPE ");
 #ifdef	XPM
   strcat(options, "XPM ");
 #endif
 #ifdef  I18N_MB
-    strcat(options, "I18N_MB ");
+  strcat(options, "I18N_MB ");
 #endif
 
   strcat(options, "M4 ");
@@ -505,6 +532,8 @@ static char *MkDef(char *name, char *def)
   else
     strcpy(cp, "define(");
 
+  if (m4_prefix_defines)
+    strcat(cp, "m4_");
   strcat(cp, name);
 
   /* Tack on "," and 2 sets of starting quotes */
