@@ -235,7 +235,10 @@ int UpdateButton(ButtonArray *array, int butnum, char *title, int up)
       temp->tw=XTextWidth(ButtonFont,title,strlen(title));
       temp->truncatewidth = 0;
     }
-    if (up!=-1) temp->up=up;
+    if (up!=-1)
+    {
+      temp->up=up;
+    }
   } else return -1;
   temp->needsupdate=1;
   return 1;
@@ -296,7 +299,7 @@ int UpdateButtonSet(ButtonArray *array, int butnum, int set)
 /******************************************************************************
   UpdateButtonDeskFlags - Change desk and flags of a button
 ******************************************************************************/
-int UpdateButtonDeskFlags(ButtonArray *array, int butnum, long desk, 
+int UpdateButtonDeskFlags(ButtonArray *array, int butnum, long desk,
 			  int is_sticky, int skip)
 {
   Button *btn;
@@ -398,6 +401,7 @@ void DoButton(Button *button, int x, int y, int w, int h, Bool clear_bg)
   /* The margin we want between the relief/text/pixmaps */
 #define INNER_MARGIN 2
 
+  h += 1;
   up=button->up;
   set=button->set;
   topgc = up ? hilite[set] : shadow[set];
@@ -492,7 +496,7 @@ void DoButton(Button *button, int x, int y, int w, int h, Bool clear_bg)
 #endif
 
   /* Draw relief last */
-  RelieveRectangle(dpy,win,x,y,w-1,h,topgc,bottomgc,button->reliefwidth);
+  RelieveRectangle(dpy,win,x,y,w-1,h-1,topgc,bottomgc,button->reliefwidth);
 
   button->needsupdate=0;
 }
@@ -557,15 +561,18 @@ void SwitchButton(ButtonArray *array, int butnum)
   Button *btn;
 
   btn = find_n(array, butnum);
-  btn->up =!btn->up;
-  btn->needsupdate=1;
-  DrawButtonArray(array, False, True);
+  if (btn != NULL)
+  {
+    btn->up =!btn->up;
+    btn->needsupdate=1;
+    DrawButtonArray(array, False, True);
+  }
 }
 
 /* -------------------------------------------------------------------------
    RadioButton - Enable button i and verify all others are disabled
    ------------------------------------------------------------------------- */
-void RadioButton(ButtonArray *array, int butnum)
+void RadioButton(ButtonArray *array, int butnum, int butnumpressed)
 {
   Button *temp;
   int i;
@@ -585,6 +592,14 @@ void RadioButton(ButtonArray *array, int butnum)
         temp->needsupdate=1;
       }
     }
+    else if (i == butnumpressed)
+    {
+      if (temp->set & 2)
+      {
+        temp->set &= 1;
+        temp->needsupdate = 1;
+      }
+    }
     else
     {
       if (ShowFocus && !temp->up)
@@ -674,7 +689,7 @@ int IsButtonVisible(Button *btn)
 }
 
 /******************************************************************************
-  IsButtonIndexVisible - Says if the button of index butnum should be 
+  IsButtonIndexVisible - Says if the button of index butnum should be
   in winlist
 ******************************************************************************/
 int IsButtonIndexVisible(ButtonArray *array, int butnum)
